@@ -1,6 +1,10 @@
 module Control.Functor.HT where
 
-import Data.Tuple.HT (fst3, snd3, thd3, )
+import qualified Data.Tuple.HT as Tuple
+import Data.Tuple.HT (fst3, snd3, thd3)
+
+import qualified Prelude as P
+import Prelude (Functor, fmap, flip, const, (.), ($), fst, snd)
 
 
 void :: Functor f => f a -> f ()
@@ -29,6 +33,19 @@ unzip3 :: Functor f => f (a, b, c) -> (f a, f b, f c)
 unzip3 x = (fmap fst3 x, fmap snd3 x, fmap thd3 x)
 
 
+{- |
+Caution: See 'unzip'.
+-}
+uncurry :: Functor f => (f a -> f b -> g) -> f (a, b) -> g
+uncurry f = P.uncurry f . unzip
+
+{- |
+Caution: See 'unzip'.
+-}
+uncurry3 :: Functor f => (f a -> f b -> f c -> g) -> f (a, b, c) -> g
+uncurry3 f = Tuple.uncurry3 f . unzip3
+
+
 mapFst :: Functor f => (a -> f c) -> (a, b) -> f (c, b)
 mapFst f ~(a,b) = fmap (flip (,) b) $ f a
 
