@@ -1,6 +1,8 @@
 module Data.Function.HT.Private where
 
-import Data.List (genericReplicate, )
+import Data.List (genericReplicate, unfoldr)
+import Data.Maybe.HT (toMaybe)
+import Data.Tuple.HT (swap)
 
 {- |
 Compositional power of a function,
@@ -16,10 +18,6 @@ nest n f x = f (nest (n-1) f x)
 nest1 n f = foldr (.) id (replicate n f)
 nest2 n f x = iterate f x !! n
 
-propNest :: (Eq a) => Int -> (a -> a) -> a -> Bool
-propNest n f x =
-   nest n f x == nest1 n f x
-
 
 {- |
 @powerAssociative@ is an auxiliary function that,
@@ -32,16 +30,20 @@ but applies "op" O(log n) times and works for large n.
 -}
 
 {-# INLINE powerAssociative #-}
-{-# INLINE powerAssociative1 #-}
-powerAssociative, powerAssociative1 ::
+powerAssociative, powerAssociativeList, powerAssociativeNaive ::
    (a -> a -> a) -> a -> a -> Integer -> a
-powerAssociative _  a0 _ 0 = a0
-powerAssociative op a0 a n =
-   powerAssociative op
-      (if even n then a0 else (op a0 a))
-      (op a a) (div n 2)
+powerAssociative op =
+   let go acc _ 0 = acc
+       go acc a n = go (if even n then acc else op acc a) (op a a) (div n 2)
+   in  go
 
-powerAssociative1 op a0 a n =
+powerAssociativeList op a0 a n =
+   foldl (\acc (bit,pow) -> if bit==0 then acc else op acc pow) a0 $
+   zip
+      (unfoldr (\k -> toMaybe (k>0) $ swap $ divMod k 2) n)
+      (iterate (\pow -> op pow pow) a)
+
+powerAssociativeNaive op a0 a n =
    foldr op a0 (genericReplicate n a)
 
 
