@@ -68,6 +68,12 @@ splitAt nt xt =
       (_, xs) -> ([],xs)
 
 
+takeRev :: [b] -> [a] -> [a]
+takeRev ys xs = drop (drop ys xs) xs
+
+dropRev :: [b] -> [a] -> [a]
+dropRev ys xs = take (drop ys xs) xs
+
 {- |
 Check whether two lists with different element types have equal length.
 It is equivalent to @length xs == length ys@ but more efficient.
@@ -124,7 +130,7 @@ shorterList xs ys =
    in  zipWith (if' useX) xs ys
 
 {- |
-This lazier than 'shorterList' in a different aspect:
+This is lazier than 'shorterList' in a different aspect:
 It returns a common prefix
 even if it is undefined, which list is the shorter one.
 However, it requires a proper 'Eq' instance
@@ -136,6 +142,9 @@ shorterListEq xs ys =
    in  zipWith (\x y -> if' (x==y || useX) x y) xs ys
 
 
+{- |
+Specialisation of 'Data.Functor.$>'.
+-}
 replicate :: [a] -> b -> [b]
 replicate xs y =
    take xs (repeat y)
