@@ -3,10 +3,13 @@ module Test.Data.List where
 import qualified Data.List.Reverse.StrictElement as Rev
 import qualified Data.List.HT.Private as ListHT
 import qualified Data.List as List
+import Data.Maybe.HT (toMaybe, )
 import Control.Monad (liftM2, )
 
+import qualified Test.QuickCheck.Modifiers as Mod
+import qualified Test.QuickCheck as QC
 import Test.Utility (equalLists, equalInfLists, )
-import Test.QuickCheck (Testable, Property, quickCheck, (==>), )
+import Test.QuickCheck (Arbitrary, Testable, Property, quickCheck, )
 
 import Prelude hiding (iterate, )
 
@@ -63,33 +66,56 @@ breakAfterUntil p xs =
    ListHT.takeUntil p xs == fst (ListHT.breakAfter p xs)
 
 
-sieve :: Eq a => Int -> [a] -> Property
-sieve n x =
-   n>0 ==>
-      equalLists [ListHT.sieve    n x,
-                  ListHT.sieve'   n x,
-                  ListHT.sieve''  n x,
-                  ListHT.sieve''' n x]
+geMaybe :: Float -> Float -> Maybe Integer
+geMaybe x y = toMaybe (x < y) (round y)
+
+dropWhileNothing :: Float -> [Float] -> Bool
+dropWhileNothing x xs =
+   ListHT.dropWhileNothing (geMaybe x) xs
+   ==
+   ListHT.dropWhileNothingRec (geMaybe x) xs
+
+dropWhileNothingBreakJust :: Float -> [Float] -> Bool
+dropWhileNothingBreakJust x xs =
+   snd (ListHT.breakJust (geMaybe x) xs)
+   ==
+   ListHT.dropWhileNothing (geMaybe x) xs
+
+breakJustRemoveEach :: Float -> [Float] -> Bool
+breakJustRemoveEach x xs =
+   ListHT.breakJust (geMaybe x) xs == ListHT.breakJustRemoveEach (geMaybe x) xs
+
+breakJustPartial :: Float -> [Float] -> Bool
+breakJustPartial x xs =
+   ListHT.breakJust (geMaybe x) xs == ListHT.breakJustPartial (geMaybe x) xs
 
 
-sliceHorizontal :: Eq a => Int -> [a] -> Bool
-sliceHorizontal n0 x =
-   let n = 1 + mod n0 1000
-   in  ListHT.sliceHorizontal n x == ListHT.sliceHorizontal' n x
+sieve :: Eq a => Mod.Positive Int -> [a] -> Bool
+sieve (Mod.Positive n) x =
+   equalLists $
+      (ListHT.sieve    n x) :
+      (ListHT.sieve'   n x) :
+      (ListHT.sieve''  n x) :
+      (ListHT.sieve''' n x) :
+      []
 
 
-sliceVertical :: Eq a => Int -> [a] -> Property
-sliceVertical n x =
-   n>0 ==>
-      ListHT.sliceVertical n x == ListHT.sliceVertical' n x
+sliceHorizontal :: Eq a => [a] -> Property
+sliceHorizontal x =
+   QC.forAll (QC.choose (1,1000)) $ \n ->
+      ListHT.sliceHorizontal n x == ListHT.sliceHorizontal' n x
 
-slice :: Eq a => Int -> [a] -> a -> Bool
-slice n0 as a =
-   let x = a:as
-       n = 1 + mod n0 (length x)
-   in  -- problems: ListHT.sliceHorizontal 4 [] == [[],[],[],[]]
-       ListHT.sliceHorizontal n x == List.transpose (ListHT.sliceVertical  n x)  &&
-       ListHT.sliceVertical  n x == List.transpose (ListHT.sliceHorizontal n x)
+
+sliceVertical :: Eq a => Mod.Positive Int -> [a] -> Bool
+sliceVertical (Mod.Positive n) x =
+   ListHT.sliceVertical n x == ListHT.sliceVertical' n x
+
+slice :: Eq a => Mod.NonEmptyList a -> Property
+slice (Mod.NonEmpty x) =
+   QC.forAll (QC.choose (1, length x)) $ \n ->
+      -- problems: ListHT.sliceHorizontal 4 [] == [[],[],[],[]]
+      ListHT.sliceHorizontal n x == List.transpose (ListHT.sliceVertical  n x)  &&
+      ListHT.sliceVertical  n x == List.transpose (ListHT.sliceHorizontal n x)
 
 
 
@@ -105,6 +131,18 @@ outerProduct xs ys =
    concat (ListHT.outerProduct (,) xs ys)  ==  liftM2 (,) xs ys
 
 
+lengthAtLeast :: Int -> [a] -> Bool
+lengthAtLeast n xs =
+   ListHT.lengthAtLeast n xs  ==  (length xs >= n)
+
+lengthAtMost :: Int -> [a] -> Bool
+lengthAtMost n xs =
+   ListHT.lengthAtMost n xs  ==  (length xs <= n)
+
+lengthAtMost0 :: Int -> [a] -> Bool
+lengthAtMost0 n xs =
+   ListHT.lengthAtMost0 n xs  ==  (length xs <= n)
+
 
 iterate :: Eq a => (a -> a -> a) -> a -> Bool
 iterate op a =
@@ -124,8 +162,8 @@ mapAdjacentPointfree xs =
 
 
 simple ::
-   (Testable test) =>
-   (Int -> [Integer] -> test) -> IO ()
+   (Show int, Arbitrary int, Testable test) =>
+   (int -> [Integer] -> test) -> IO ()
 simple = quickCheck
 
 elemCheck ::
@@ -148,12 +186,21 @@ tests =
    ("breakAfter1",      elemCheck (\a -> breakAfter1 (a>=))) :
    ("breakAfter2",      elemCheck (\a -> breakAfter2 (a>=))) :
    ("breakAfterUntil",  elemCheck (\a -> breakAfterUntil (a>=))) :
+   ("dropWhileNothing", elemCheck dropWhileNothing) :
+   ("dropWhileNothingBreakJust",
+                        elemCheck dropWhileNothingBreakJust) :
+   ("breakJustRemoveEach",
+                        elemCheck breakJustRemoveEach) :
+   ("breakJustPartial", elemCheck breakJustPartial) :
    ("sieve",            simple sieve) :
-   ("sliceHorizontal",  simple sliceHorizontal) :
+   ("sliceHorizontal",  quickCheck (sliceHorizontal :: String -> Property)) :
    ("sliceVertical",    simple sliceVertical) :
-   ("slice",            simple slice) :
+   ("slice",            quickCheck (slice :: Mod.NonEmptyList Char -> Property)) :
    ("shear",            quickCheck (shear           :: [[Integer]] -> Bool)) :
    ("outerProduct",     quickCheck (outerProduct    :: [Integer] -> [Int] -> Bool)) :
+   ("lengthAtLeast",    simple lengthAtLeast) :
+   ("lengthAtMost",     simple lengthAtMost) :
+   ("lengthAtMost0",    simple lengthAtMost0) :
    ("iterate",          quickCheck (iterate (+)     :: Integer -> Bool)) :
    ("mapAdjacent",      quickCheck (mapAdjacent     :: Integer -> [Integer] -> Bool)) :
    ("mapAdjacentPointfree",
